#include <ChallengerWiFi.h>
#include <AT.h>
#include <EspATMQTT.h>
#include <WiFiEspAT.h>
#include <ArduinoJson.h>
#include <Adafruit_NeoPixel.h>
#include <MC34X9.h>

#define USE_I2C       1
#define I2C_ADDRESS   0x4C
#define USE_SPI       0

// MQTT settings
EspATMQTT mqtt;
const char *mqtt_server = "192.168.1.6";
const char *mqttChannelOut = "ChallengerRP2040/out";
const char *mqttChannelIn =  "ChallengerRP2040/in";

// WiFi access point settings
const char ssid[] = "ssid"; 
const char pass[] = "password";

// Onboard neopixel settings
const int NumNeoPixels = 1;
Adafruit_NeoPixel pixels(NumNeoPixels, NEOPIXEL, NEO_GRB + NEO_KHZ800);

// Motion settings
const bool enableTILT = false;
const bool enableFLIP = false;
const bool enableANYM = false;
const bool enableSHAKE = true;
const bool enableTILT_35 = false;

MC34X9 MC34X9_acc = MC34X9();

void mqtt_topic_cb(char *topic, char *mqttdata) {
  DynamicJsonDocument json(1024);

  Serial.print(topic);
  Serial.print(" - ");
  Serial.println(mqttdata);

  deserializeJson(json, mqttdata);
  int r = json["r"];
  int g = json["g"];
  int b = json["b"];

  pixels.clear();
  pixels.setPixelColor(0, pixels.Color(r, g, b));
  pixels.show();
}

void setup() {

  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, 0);
  pixels.begin();
  pixels.clear();
  pixels.setPixelColor(0, pixels.Color(0, 0, 0));
  pixels.show();

  while (!Serial) {
    delay(10);
  }
  Serial.begin(115200);
  Serial.println("iLabs Challenger RP2040 Wifi demo");

  if (Challenger2040WiFi.reset())
      Serial.println(F("WiFi Chip reset OK !"));
  else
      Serial.println(F("Could not reset WiFi chip !"));

  WiFi.init(Serial2);

  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println();
    Serial.println("Communication with WiFi module failed!");
    while (true);
  }

  Serial.print("WiFi connecting to "); Serial.println(ssid);
  int status = WiFi.begin(ssid, pass);
  Serial.print("WiFi status: "); Serial.println(status);

  Serial.println("Setting up MQTT");
  mqtt.begin();
  mqtt.userConfig(DEFAULT_LINK_ID, ESP_MQTT_SCHEME_MQTT_OVER_TCP, "Challenger_RP2040_WiFi");
  delay(1000);
  mqtt.connect(DEFAULT_LINK_ID, mqtt_server);
  mqtt.subscribeTopic(mqtt_topic_cb, DEFAULT_LINK_ID, mqttChannelIn);

  Serial.println("Setting up accelerometer");
  if (!MC34X9_acc.begin(USE_I2C, I2C_ADDRESS)) {
    Serial.println("No MC34X9");
    while (true);
  }
  MC34X9_acc.stop();
  MC34X9_acc.SetSampleRate(MC34X9_SR_DEFAULT_1000Hz);
  MC34X9_acc.SetMotionCtrl(enableTILT, enableFLIP, enableANYM, enableSHAKE, enableTILT_35);
  MC34X9_acc.SetINTCtrl(enableTILT, enableFLIP, enableANYM, enableSHAKE, enableTILT_35);
  MC34X9_acc.wake();
  Serial.println("Motion sensor enabled");
}

bool checkMotionInterrupt() {
  bool retCode = false;
  MC34X9_interrupt_event_t evt_mc34X9 = {0};
  MC34X9_acc.INTHandler(&evt_mc34X9);

  uint8_t* iter = (uint8_t*) &evt_mc34X9;
  for (int i = 0; i < sizeof (MC34X9_interrupt_event_t); i++, iter++) {
    if ((*iter) & 0x01)
      retCode = true;
  }

  if (retCode) {
    Serial.print("Motion detected: ");
  }

  if (evt_mc34X9.bTILT) {
    Serial.print("'Tilt' ");
  }
  if (evt_mc34X9.bFLIP) {
    Serial.print("'Flip' ");
  }
  if (evt_mc34X9.bANYM) {
    Serial.print("'Any' ");
  }
  if (evt_mc34X9.bSHAKE) {
    Serial.print("'Shake' ");
  }
  if (evt_mc34X9.bTILT_35) {
    Serial.print("'Tilt 35' ");
  }
 
  if (retCode) {
    Serial.println();
  }

  return retCode;
}

void dumpRawAcceleratorValues() {

  MC34X9_acc_t rawAccel = MC34X9_acc.readRawAccel();

  Serial.print("aX: \t"); Serial.print(rawAccel.XAxis_g); Serial.print("\t");
  Serial.print("aY: \t"); Serial.print(rawAccel.YAxis_g); Serial.print("\t");
  Serial.print("aZ: \t"); Serial.print(rawAccel.ZAxis_g); Serial.print("\t");
  Serial.println("m/s^2");
}

void publishGetRandomColorCommand() {
  StaticJsonDocument<1024> doc;
  char output[1024];

  if (!mqtt.isConnected()) {
    Serial.println("MQTT is not connected");
    return;
  }

  Serial.println("Send getRandomColor command to mqtt broker");

  doc["command"] = "getRandomColor";
  serializeJson(doc, output);
  mqtt.pubRaw(DEFAULT_LINK_ID, mqttChannelOut, output);
}

void blink() {
  static uint32_t timer = millis();
  static uint32_t delta = 5000;
  static bool led = false;

  if (millis() - timer > delta) {
    timer = millis();
    if (delta == 5000) {
      delta = 500;
      led = true;
    } else {
      delta = 5000;
      led = false;
    }
    digitalWrite(LED_BUILTIN, led);
  }
}

void loop() {
  blink();
  mqtt.process();

  if (checkMotionInterrupt()) {
      dumpRawAcceleratorValues();
      publishGetRandomColorCommand();
  }

}
